#
# This file is part of openCARP
# (see https://www.openCARP.org).
#
# The openCARP project licenses this file to you under the
# Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#

"""
Bundle module, implementing the --bundle option

Use of the --bundle option:

    run.py --bundle exp_name

where exp_name is the name of the experiment.
If it is not specified, it is set to 'experiment' by default.

This option creates a self-contained bundle of the experiment which is run,
if the following constraints are followed:
in the run.py script, every path to an external file used in the experiment should
be encapsulated in the carputils.tools.simfile_path function.
Plus, if the file is a mesh file, the 'mesh' flag in the simfile_path function should
be set to True.

Note that additional files can be added to the bundle manually:
- if there is a folder named 'external' in run.py's directory, its content will be added
to the bundle automatically.
- if there is a folder named 'Mesh' in run.py's directory, its content will be added
to the bundle automatically.
- If the folder carputils.settings.config.MESH_DIR/exp_name/ exists, then its content will be added
to the bundle automatically.

Some metadata is also shared in the bundle: at the root of the bundle, a file named METADATA.yml
is created. This file is filled with metadata provided by the user in a file named METADATA.yml
(preferably using the template located at carputils/resources/METADATA.yml.tpl); it will also
contain autogenerated data about the environment in which the bundle was created:
- information about the creator found in carputils settings file
- the name given to the bundle
- the command line to use to run the experiment
- the python version used to create the bundle

It is also possible to upload the experiment bundle on a (non-existing or empty) git repository.
In order to do so, the '--push-bundle' option has to be used additionally.
Example:
    run.py --bundle exp_name --push-bundle https://gitlab.com/username/my-project.git
It is also possible to provide the link to the git repository in METADATA.yml, under the 'bundle_repository'
field (and then no argument should be passed to the '--push-bundle' option)

Additionally to the '--push-bundle' option, the '--release-bundle' option can be used in order to ask for a
release of the experiment on opencarp.org.
Example:
    run.py --bundle exp_name --push-bundle https://gitlab.com/username/my-project.git --release-bundle
By using this option, the email client of the user is opened, containing the email that has to be sent in
order to publish the experiment on the openCARP website.

It is also possible to provide the outputs of the simulation in the bundle, by using the
`--bundle-output` option in addition to the `--bundle` option.
This option creates a `sim_outputs` folder in the bundle, in which is put:
- the `meshes` folder located in the experiment directory
- the `$jobID` folder which contains the outputs of the experiment
"""

import sys
from carputils.bundle.bundle import BundleModule

# Replace this module with the Bundle instance
# This hack is 'officially' supported, as outlined in this message by Gudio van
# Rossum, the creator of Python:
#       https://mail.python.org/pipermail/python-ideas/2012-May/014969.html
if hasattr(sys.modules['__main__'], '__file__'):
    sys.modules[__name__] = BundleModule()